package provide objLab 0.3

#Módulo objLab.tcl versão 03 do programa Orion

#Programa para controle de equipamento analítico baseado na filosofia
#de orientação à objeto

#classe sensor
#objeto eletrodo_pH
#objeto eletrodo_ORP
#objeto eletrodo_OD

#*******************************************************************
#Cada novo atributo é armazenado em um array para o qual devem
#ser criados os respectivos métodos para atualizar (def_...)
#e resgatar o valor atual (get_...)
#Os novos métodos devem ser inseridos na rotina cria_obj
#E na rotima remover_sensor deve ser inserida os atributos
#para serem liberados da memória
#*******************************************************************

#*******************************************************************
#Atributos dos objetos da classe sensor
#unidade é um array global que vai guardar o valor do atributo
#unidade do parâmetro e será indexado pelo nome do objeto
#e armazenando como valor a unidade de concentração 
#*******************************************************************

global unidade



#*******************************************************************
#estado_sensor é um array global que vai guardar o valor do atributo
#estado do sensor e será indexado pelo nome do objeto
#e poderá armazenar como valor uma das opções: monitorando, 
#parado, pausado, calibrando, condicionando
#*******************************************************************

global estado_sensor



#*******************************************************************
#Leitura é um array global que vai guardar o valor do atributo
#última_leitura do sensor e será indexado pelo nome do objeto
#armazenando como valor uma lista com o formato {tempo leitura}
#*******************************************************************

global leitura



#*******************************************************************
#procedimento para retornar a unidade usada pelo parâmetro associado
#ao respectivo sensor
#*******************************************************************

proc get_unidade { nome_sensor } {
    
    global unidade
    
    if { [info exists unidade($nome_sensor)] } {
	
	return $unidade($nome_sensor)
	
    } else {
	
	puts "Atenção: $nome_sensor não tem unidade"
	return "unidade_indefinida"
	
    }
    
}


#*******************************************************************
#Procedimento para definir a unidade do parâmetro associado ao sensor
#*******************************************************************

proc def_unidade { nome_sensor valor_unidade } {

global unidade

set unidade($nome_sensor) $valor_unidade

}


#*******************************************************************
#procedimento para retornar o estado atual do sensor 
#*******************************************************************

proc get_estado_sensor { nome_sensor } {
    
    global estado_sensor
    
    if { [info exists estado_sensor($nome_sensor)] } {
	    
	    return $estado_sensor($nome_sensor)
	    
	} else {
	    
	    puts "Atenção: $nome_sensor não tem estado definido"
	    return "estado_sensor_indefinido"
	    
	}
	
}

#*******************************************************************
#Procedimento para definir o estado atual do sensor
#*******************************************************************

proc def_estado_sensor { nome_sensor valor_estado_sensor } {

global estado_sensor

set estado_sensor($nome_sensor) $valor_estado_sensor

}


#*******************************************************************
#Procedimento para definir a ultima leitura do sensor
#*******************************************************************

proc def_leitura { nome_sensor valor_leitura } {

global leitura

set leitura($nome_sensor) $valor_leitura

}


#*******************************************************************
#procedimento para retornar a leitura atual do sensor 
#*******************************************************************

proc get_leitura { nome_sensor } {
    
    global leitura
    
    if { [info exists leitura($nome_sensor)] } {
	
	return $leitura($nome_sensor)
	
    } else {
	
	puts "Atenção: $nome_sensor não tem leitura"
	return "leitura_indefinida"
	
    }
    
}





#*******************************************************************
#Processo de criação de um objeto da classe sensor
#O argumento "args" é um nome especial que armazena todos 
#os argumentos restantes em uma lista de tamanho variável
#*******************************************************************


proc cria_sensor { nome_sensor comando args } {
    if { $comando == "get_unidade" } {
	    return [get_unidade $nome_sensor]
	} elseif { $comando == "def_unidade"} {
	    def_unidade $nome_sensor [lindex $args 0]
	} elseif { $comando == "get_estado_sensor" } {
	    return [get_estado_sensor $nome_sensor]
	} elseif { $comando == "def_estado_sensor"} {
	    def_estado_sensor $nome_sensor [lindex $args 0]
	} elseif { $comando == "get_leitura" } {
	    return [get_leitura $nome_sensor]
	} elseif { $comando == "def_leitura"} {
	    def_leitura $nome_sensor [lindex $args 0]
	} else {
	    puts "Erro: $comando é um comando desconhecido"
	}
}

#*******************************************************************
#Procedimento que cria a classe sensor
#*******************************************************************

proc sensor {args} {
    foreach nome $args {
#	puts "Criando sensor $nome"
	proc $nome {comando args} "return \[eval cria_sensor $nome \$comando \$args\]"
    }
}


proc remover_sensor { args } {
    
    global unidade
    global estado_sensor
    global leitura
    
    foreach nome $args {
	if { [info exists unidade($nome)] } {
		unset unidade($nome)  ;#Deleta os dados do objeto
	    }
	    if { [info exists estado_sensor($nome)] } {
		    unset estado_sensor($nome)   ;#Deleta os dados do objeto
		}
		if { [info exists leitura($nome)] } {
			unset leitura($nome) ;#Deleta os dados do objeto
		    }
		    rename $nome {}       ;#Deleta o comando objeto
    }
}





#***************************************************************************************
#A partir daqui, vamos criar os procedimentos da classe amostra
#a qual vai possuir os seguintes atributos/propriedades:
#projeto {codigo coordenador}
#experimento {numero estado} estado (iniciando andamento pausado interrompido realizado)
#coleta {{local} data}
#responsavel_experimento
#parametros_monitorados {pH lim_inf lim_sup ORP lim_inf lim_sup etc}
#**************************************************************************************


#*******************************************************************
#Atributos dos objetos da classe amostra
#durante um projeto serão realizados vários experimentos
#e cada experimento irá gerar várias amostras
#que serão criadas como objetos e descartadas ao final de 
#cada análise, cujo nome será o próprio código da amostra
#Ou seja, o nome de cada objeto "amostra" será o próprio código da amostra
#As variáveis a seguir serão arrays globais que vão guardar os 
#atributos das amostras e será indexado pelo codigo de cada amostra
#Será usada a hora do sistema no formato original como código de
#identificação de cada amostra
#Quanto ao responsável pelo experimento seria mais conveniente usar a chave para
#padronizar o armazenamento e a busca
#*******************************************************************

#projeto {codigo coordenador}
#experimento {numero estado_atual (iniciando andamento pausado interrompido realizado)}
#coleta {local data responsavel}
#responsavel_experimento
#parametros_monitorados {p1 lim_inf lim_sup p2 lim_inf lim_sup p3 lim_inf lim_sup ...}

#O array data_coleta é desnessário pois o codigo da amostra já
#armazena as informações de dia e hora da coleta
global projeto
global experimento 
global coleta
global responsavel_experimento
global parametros_monitorados


#*******************************************************************
#Deixamos explícito os dois argumentos para que o retorno 
#das informações do projeto fossem no formato de lista
#*******************************************************************

proc def_projeto { cod_amostra codigo coordenador } {
    
    global projeto
    
    set projeto($cod_amostra) [list $codigo $coordenador]
    
}


proc get_projeto { cod_amostra } {
    
    global projeto
    
    if { [info exists projeto($cod_amostra)] } {
	    
	    return $projeto($cod_amostra)
	    
	} else {
	    
	    puts "Atenção:$cod_amostra não tem projeto associado"
	    return "sem_projeto_associado"
	}
}


#*******************************************************************
#Deixamos explícito os dois argumentos para que o retorno 
#das informações do experimento fossem no formato de lista
#*******************************************************************


proc def_experimento { cod_amostra codigo estado } {
    
    global experimento
    
    set experimento($cod_amostra) [list $codigo $estado]
    
}

proc get_experimento { cod_amostra } {
    
    global experimento
    
    if { [info exists experimento($cod_amostra)] } {
	    return $experimento($cod_amostra)
	    
	} else {
	    
	    puts "Atenção: $cod_amostra não tem experimento associado"
	    return "sem_experimento_associado"
	}
}

#*******************************************************************
#Neste caso foi incluido apenas o local de amostragem pois o código
#da amostra já contém as informações de data
#*******************************************************************


proc def_coleta { cod_amostra local data responsavel} {
    
    global coleta
    
    set coleta($cod_amostra) [list $local $data $responsavel]
    
}

proc get_coleta { cod_amostra } {
    
    global coleta
    
    if { [info exists coleta($cod_amostra)] } {
	    return $coleta($cod_amostra)
	    
	} else {
	    
	    puts "Atenção: $cod_amostra não tem local  e data associado"
	    return "local_coleta_indefinido"
	}
}


proc def_responsavel_experimento { cod_amostra resp_exp } {
    
    global responsavel_experimento 
    
    set responsavel_experimento($cod_amostra) $resp_exp
    
}


proc get_responsavel_experimento { cod_amostra } {
    
    global responsavel_experimento 
    
    if { [info exists responsavel_experimento($cod_amostra)] } {
	   
	    return $responsavel_experimento($cod_amostra)
	    
	} else {
	    
	    puts "Atenção:$cod_amostra não tem um responsavel associado"
	    return "sem_responsavel"
	}
}


#*******************************************************************
#Neste caso o número de parâmetros é variável 
#por isso deixamos aberto usando o argumento "args"
#Incluimos o comando lindex 2 vezes para incluir
#apenas uma lista de N elementos dispensando
#posterior tratamento dos dados retornados pela função get_
#*******************************************************************


proc def_parametros_monitorados { cod_amostra args } {
    
    global parametros_monitorados 
    

    set parametros_monitorados($cod_amostra) [lindex [lindex $args 0] 0]
}

proc get_parametros_monitorados { cod_amostra } {
    
    global parametros_monitorados 
    
    if { [info exists parametros_monitorados($cod_amostra)] } {
	   
	return $parametros_monitorados($cod_amostra)
	    
	} else {
	    
	    puts "Atenção:$cod_amostra não tem parâmetro associado"
	    return "sem_parâmetros"
	}
}


#*******************************************************************
#Processo de criação de um objeto da classe amostra
#O argumento "args" é um nome 
#especial que armazena todos os argumentos restantes em uma lista
#*******************************************************************


proc cria_amostra { cod_amostra comando args } {

    if { $comando == "get_projeto" } {
	    return [get_projeto $cod_amostra]
	} elseif { $comando == "def_projeto"} {
	    def_projeto $cod_amostra [lindex $args 0] [lindex $args 1]

	} elseif { $comando == "get_experimento" } {
	    return [get_experimento $cod_amostra]
	} elseif { $comando == "def_experimento"} {
	    def_experimento $cod_amostra [lindex $args 0] [lindex $args 1]

	} elseif { $comando == "get_coleta" } {
	    return [get_coleta $cod_amostra]
	} elseif { $comando == "def_coleta"} {
	    def_coleta $cod_amostra [lindex $args 0] [lindex $args 1] [lindex $args 2]

	} elseif { $comando == "get_responsavel_experimento" } {
	    return [get_responsavel_experimento $cod_amostra]
	} elseif { $comando == "def_responsavel_experimento"} {
	    def_responsavel_experimento $cod_amostra [lindex $args 0]

	} elseif { $comando == "get_parametros_monitorados" } {
	    return [get_parametros_monitorados $cod_amostra]
	} elseif { $comando == "def_parametros_monitorados"} {
	    def_parametros_monitorados $cod_amostra $args

	} else {
	    puts "Erro: $comando é um comando desconhecido"
	}
}

#*******************************************************************
#Procedimento que cria a classe amostra
#*******************************************************************

proc amostra {args} {
    foreach codigo $args {
	proc $codigo {comando args} "return \[eval cria_amostra $codigo \$comando \$args\]"
    }
}


proc remover_amostra { args } {
    
    global projeto
    global experimento 
    global coleta
    global responsavel_experimento
    global parametros_monitorados

    
    foreach nome $args {
	if { [info exists projeto($nome)] } {
	    unset projeto($nome)  ;#Deleta os dados do objeto
	}
	if { [info exists experimento($nome)] } {
	    unset experimento($nome)   ;#Deleta os dados do objeto
	}
	if { [info exists coleta($nome)] } {
	    unset coleta($nome) ;#Deleta os dados do objeto
	}
	if { [info exists responsavel_experimento($nome)] } {
	    unset responsavel_experimento($nome) ;#Deleta os dados do objeto
	}
	if { [info exists parametros_monitorados($nome)] } {
	    unset parametros_monitorados($nome) ;#Deleta os dados do objeto
		    }
	rename $nome {}       ;#Deleta o comando objeto
    }
}

#*******************************************************************
#Procedimento de criação da classe medida
#Cada objeto da classe "medida" (measurement) vai representar 
#um resultado analítico compreendendo uma ou várias leituras, 
#com os respectivos tempos, de um determinado parâmetro vinculado 
#a uma amostra através do código da amostra
#O nome da cada instância de medida pode ser o nome do parâmetro
#Identifiquei a necessidade desta classe pois para cada amostra 
#serão geradas mais de uma medida simultâmeamente
#Essa classe poderá acomodar futuramente outros atributos tais
#como curva de calibração
#*******************************************************************

#O nome do objeto já é o nome do parâmetro monitorado
#amostra_medida { cod_amostra } ;#código da amostra
#parametro_medido ( parametro }
#leituras { tempo leitura } ;#leitura


#Arrays globais que vão armazenar os respectivos atributos
#das medidas
global amostra_medida
global parametro_medido
global parametro_unidade
global leituras

proc def_amostra_medida { cod_medida amostra } {
    
    global amostra_medida
    
    set amostra_medida($cod_medida) $amostra
    
}


proc get_amostra_medida { cod_medida } {
    
    global amostra_medida
    
    if { [info exists amostra_medida($cod_medida)] } {
	
	return $amostra_medida($cod_medida)
	
    } else {
	
	puts "Atenção $cod_medida não tem uma amostra associada"
	return "sem_amostra"
    }
}


proc def_parametro_medido { cod_medida parametro } {
    
    global parametro_medido
    
    set parametro_medido($cod_medida) $parametro
    
}



proc get_parametro_medido { cod_medida } {
    
    global parametro_medido
    
    if { [info exists parametro_medido($cod_medida)] } {
	
	return $parametro_medido($cod_medida)
	
    } else {
	
	puts "Atenção $cod_medida não tem parâmetro medido"
	
    }
}


proc def_parametro_unidade { cod_medida unidade } {
    
    global parametro_unidade
    
    set parametro_unidade($cod_medida) $unidade
    
}



proc get_parametro_unidade { cod_medida } {
    
    global parametro_unidade
    
    if { [info exists parametro_unidade($cod_medida)] } {
	
	return $parametro_unidade($cod_medida)
	
    } else {
	
	puts "Atenção $cod_medida não tem unidade associada"
	
    }
}


proc def_leituras { cod_medida args } {
    
    global leituras   

    set leituras($cod_medida) [lindex $args 0]
    
}

proc get_leituras { cod_medida } {
    
    global leituras
    
    if { [info exists leituras($cod_medida)] } {
	return $leituras($cod_medida)
	
    } else {
	
	puts "Atenção $cod_medida não tem leitura"
	
    }
    
}

#*******************************************************************
#Processo de criação de um objeto da classe "medida"
#*******************************************************************

proc cria_medida { cod_medida comando args } {

    if { $comando == "get_amostra_medida" } {
	    return [get_amostra_medida $cod_medida]

	} elseif { $comando == "def_amostra_medida"} {
	    def_amostra_medida $cod_medida [lindex $args 0]

	} elseif { $comando == "get_parametro_medido" } {
	    return [get_parametro_medido $cod_medida]

	} elseif { $comando == "def_parametro_medido"} {
	    def_parametro_medido $cod_medida [lindex $args 0]

	} elseif { $comando == "get_parametro_unidade" } {
	    return [get_parametro_unidade $cod_medida]

	} elseif { $comando == "def_parametro_unidade"} {
	    def_parametro_unidade $cod_medida [lindex $args 0]

	} elseif { $comando == "get_leituras" } {
	    return [get_leituras $cod_medida]

	} elseif { $comando == "def_leituras"} {
	    def_leituras $cod_medida $args

	} else {
	    puts "Erro: $comando é um comando desconhecido"
	}
}


#*******************************************************************
#Procedimento que cria a classe medida
#*******************************************************************

proc medida {args} {
    foreach codigo $args {
	proc $codigo {comando args} "return \[eval cria_medida $codigo \$comando \$args\]"
    }
}


#*******************************************************************
#Procedimento para remover os objetos da classe medida
#*******************************************************************


proc remover_medida { args } {
    
    global amostra_medida
    global parametro_medido
    global parametro_unidade
    global leituras
    
    
    foreach nome $args {
	if { [info exists amostra_medida($nome)] } {
	    unset amostra_medida($nome)  ;#Deleta os dados do objeto
	}
	if { [info exists parametro_medido($nome)] } {
	    unset parametro_medido($nome)   ;#Deleta os dados do objeto
	}
	if { [info exists parametro_unidade($nome)] } {
	    unset parametro_unidade($nome)   ;#Deleta os dados do objeto
	}	
	if { [info exists leituras($nome)] } {
	    unset leituras($nome) ;#Deleta os dados do objeto
	}
	rename $nome {}       ;#Deleta o comando objeto
    }
}

